/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.22                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_SH7203.c for RENESAS SH2A SH7203
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef OS_FSYS
  /* Internal MTU clock frequency                                   */
  /* Depends on external oscillator and PLL settings used           */
  /* We assume Clock mode 0 with 16.66 MHz oscillator at EXTAL      */
  /* The PLL is initialized in the function                         */
  /* __low_level_init()                                             */
  #define OS_FSYS (200000000L)  /* Standard setting is 200 MHz      */
#endif

#ifndef   OS_PCLK_DIVIDER
  #define OS_PCLK_DIVIDER  (6)
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER (OS_FSYS / OS_PCLK_DIVIDER / 8)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART (OS_FSYS / OS_PCLK_DIVIDER)
#endif

#ifndef   OS_INIT_PLL
  #define OS_INIT_PLL (1)
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif


#ifndef   OS_VIEW_USE_UART          // If set, UART will be used for communication
  #define OS_VIEW_USE_UART  (1)     // Default: 1 => Uart is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UART0 and UART1 are supported and the standard
*       setup enables UART 0 per default
*       When using DCC for communiction, the UART is not used for embOSView,
*       regardless the OS_UART settings
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   ALLOW_NESTED_INTERRUPTS
  #define ALLOW_NESTED_INTERRUPTS (0)
#endif

/******	End of configurable option **********************************/

// Only UART 0 was tested, other UART may require modification
#define OS_UART_USED (OS_VIEW_ENABLE && (OS_UART == 0))

#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

#define OS_TIMER_RELOAD (OS_PCLK_TIMER / 1000u)

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/

#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/
#define CCR1    (*(unsigned int*)0xFFFC1000u)  // Cache Control Register 1 (CCR1)

/****** Standby control register ************************************/
#define STB_BASE_ADDR          (0xFFFE0000u)
#define STBCR4                 (*(volatile OS_U8* ) (STB_BASE_ADDR + 0x040Cu))

/* Compare match timer */
#define CMT_BASE_ADDR          (0xFFFEC000u)
#define CMSTR                  (*(volatile OS_U16*) (CMT_BASE_ADDR + 0x00u))
#define CMCSR_0                (*(volatile OS_U16*) (CMT_BASE_ADDR + 0x02u))
#define CMCNT_0                (*(volatile OS_U16*) (CMT_BASE_ADDR + 0x04u))
#define CMCOR_0                (*(volatile OS_U16*) (CMT_BASE_ADDR + 0x06u))

#define CMT_CMSR0_CMF_BIT      (7u)
#define CMT_MODULE_ID_BIT      (2u)

/* Interrupt controller */
#define INTC_BASE_ADDR         (0xFFFE0800u)

#define INTC_IBCR              (*(volatile OS_U16*) (INTC_BASE_ADDR + 0x000cu))
#define INTC_IBNR              (*(volatile OS_U16*) (INTC_BASE_ADDR + 0x000eu))
#define INTC_IPR08             (*(volatile OS_U16*) (INTC_BASE_ADDR + 0x0404u))
#define INTC_IPR13             (*(volatile OS_U16*) (INTC_BASE_ADDR + 0x040Eu))
#define INTC_IPR14             (*(volatile OS_U16*) (INTC_BASE_ADDR + 0x0410u))

#define INTC_CMT0_PRIO_REG     INTC_IPR08
#define CMT0_PRIO_POS          (4u)
#define CMT0_PRIO_MASK         (~(0x0Fu << CMT0_PRIO_POS))

/****** Watchdog ****************************************************/

#define WDT_BASE_ADDR          (0xfffe0000u)
#define WDT_WTCSR              (*(volatile OS_U16*) (WDT_BASE_ADDR + 0x0000u))
#define WDT_WTCNT              (*(volatile OS_U16*) (WDT_BASE_ADDR + 0x0002u))
#define WDT_WRCSR              (*(volatile OS_U16*) (WDT_BASE_ADDR + 0x0004u))

/****** Clock Pulse Generator ***************************************/

#define CPG_FRQCR              (*(volatile OS_U16*) (0xFFFE0010u))

/****** UART ********************************************************/
#define SCIF0_BASE_ADDR        (0xFFFE8000u)
#define SCIF1_BASE_ADDR        (0xFFFE8800u)
#define SCIF2_BASE_ADDR        (0xFFFE9000u)
#define SCIF3_BASE_ADDR        (0xFFFE9800u)

#define PECRL1                 (*(volatile OS_U16*) (0xFFFE3A16u))
#define PECRL2                 (*(volatile OS_U16*) (0xFFFE3A14u))
#define PECRL4                 (*(volatile OS_U16*) (0xFFFE3A10u))
#define PEIORL                 (*(volatile OS_U16*) (0xFFFE3A06u))

#define BAUDPRESCALE 0  // CKE bits: 0,1,2,3
#define BAUDDIVIDE ((OS_PCLK_UART+(OS_BAUDRATE*16L))/(OS_BAUDRATE*32L))-1
#if (OS_UART == 0)
  #define SCIFx_MODULE_ID_BIT  (7u)
  #define SCIF_BASE_ADDR       (SCIF0_BASE_ADDR)
 #define INTC_SCIFx_PRIO_REG   (INTC_IPR13)
  #define SCIFx_PRIO_POS       (8u)
  #define PECRLx               (PECRL1)
  #define PE_MD_TXD_POS        (4u)
  #define PE_MD_RXD_POS        (0u)
  #define PE_TXD_PIN           (1u)
  #define PE_RXD_PIN           (0u)
#elif (OS_UART == 1)
  #define SCIFx_MODULE_ID_BIT  (6u)
  #define SCIF_BASE_ADDR       (SCIF1_BASE_ADDR)
  #define INTC_SCIFx_PRIO_REG  (INTC_IPR13)
  #define SCIFx_PRIO_POS       (4u)
  #define PECRLx               (PECRL2)
  #define PE_MD_TXD_POS        (4u)
  #define PE_MD_RXD_POS        (0u)
  #define PE_TXD_PIN           (5u)
  #define PE_RXD_PIN           (4u)
#elif (OS_UART == 2)
  #define SCIFx_MODULE_ID_BIT  (5u)
  #define SCIF_BASE_ADDR       (SCIF2_BASE_ADDR)
  #define INTC_SCIFx_PRIO_REG  (INTC_IPR13)
  #define SCIFx_PRIO_POS       (0u)
  #define PECRLx               (PECRL2)
  #define PE_MD_TXD_POS        (12u)
  #define PE_MD_RXD_POS        (8u)
  #define PE_TXD_PIN           (7u)
  #define PE_RXD_PIN           (6u)
#elif (OS_UART == 3)
  #define SCIFx_MODULE_ID_BIT  (4u)
  #define SCIF_BASE_ADDR       (SCIF3_BASE_ADDR)
  #define INTC_SCIFx_PRIO_REG  (INTC_IPR14)
  #define SCIFx_PRIO_POS       (12u)
  #define PECRLx               (PECRL4)
  #define PE_MD_TXD_POS        (4u)
  #define PE_MD_RXD_POS        (0u)
  #define PE_TXD_PIN           (13u)
  #define PE_RXD_PIN           (12u)
#endif

#define SCIFx_SCSMR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x00u))
#define SCIFx_SCBRR            (*(volatile OS_U8* ) (SCIF_BASE_ADDR + 0x04u))
#define SCIFx_SCSCR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x08u))
#define SCIFx_SCFTDR           (*(volatile OS_U8* ) (SCIF_BASE_ADDR + 0x0Cu))
#define SCIFx_SCFSR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x10u))
#define SCIFx_SCFRDR           (*(volatile OS_U8* ) (SCIF_BASE_ADDR + 0x14u))
#define SCIFx_SCFCR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x18u))
#define SCIFx_SCFDR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x1Cu))
#define SCIFx_SCSPTR           (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x20u))
#define SCIFx_SCLSR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x24u))
#define SCIFx_SCEMR            (*(volatile OS_U16*) (SCIF_BASE_ADDR + 0x28u))

#define SCSCR_REIE_BIT         (3u)
#define SCSCR_RE_BIT           (4u)
#define SCSCR_TE_BIT           (5u)
#define SCSCR_RIE_BIT          (6u)
#define SCSCR_TIE_BIT          (7u)

#define SCFSR_DR_BIT           (0u)
#define SCFSR_RDF_BIT          (1u)
#define SCFSR_PER_BIT          (2u)
#define SCFSR_FER_BIT          (3u)
#define SCFSR_BRK_BIT          (4u)
#define SCFSR_TDFE_BIT         (5u)
#define SCFSR_TEND_BIT         (6u)
#define SCFSR_ER_BIT           (7u)

#define PE_MDRXD_MASK          (0xFuL << PE_MD_RXD_POS)
#define PE_MDTXD_MASK          (0xFuL << PE_MD_TXD_POS)

/*********************************************************************
*
*	Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*		_IsrTickHandler()
*/
static void _IsrTickHandler(void) {
  /* Clear the interrupt flag	*/
  CMCSR_0 &= ~(1u << CMT_CMSR0_CMF_BIT);
  OS_TICK_Handle();
}

/*********************************************************************
*
*       OS_ISR_Tick(), the embOS timer interrupt handler
*/
#pragma vector = 142
static __fast_interrupt __interrupt void OS_ISR_Tick(void) {
  #if ALLOW_NESTED_INTERRUPTS
    OS_CallNestableISR(_IsrTickHandler);
  #else
    OS_CallISR(_IsrTickHandler);
  #endif
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*   Returns the current hardware timer count value
*
* Return value
*   Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return CMCNT_0;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*   Returns if the hardware timer interrupt pending flag is set
*
* Return value
*   == 0; Interrupt pending flag not set
*   != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (CMCSR_0 & (1u << 7));
}

/*********************************************************************
*
*	Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  // SysTimerConfig.TimerFreq will be set later, thus it is initialized with zero.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_FSYS, OS_TICK_FREQ, 1, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_IncDI();                       // For Safety disable interrupts

  //
  // Enable module clock for CMT and initialize CMT
  //
  STBCR4 &= ~(1u << CMT_MODULE_ID_BIT);
  CMSTR  &= ~(1u << 0);             // Stop counter channel 0
  CMCOR_0 = OS_TIMER_RELOAD;        // Initialize compare value
  CMCNT_0 = 0u;                     // Reset counter
  CMCSR_0 = 0u                      // Set divider to 1/8
          | (1u << 6);              // Compare match interrupt enable
  INTC_CMT0_PRIO_REG &= CMT0_PRIO_MASK;
  INTC_CMT0_PRIO_REG |= (1u << CMT0_PRIO_POS);         // Set interrupt priority, enable lowest

  CMSTR  |= (1u << 0);              // Start timer

  INTC_IBNR = 0u
             | (1u << 14)           // Enable Bank switching for all interrupts
             | (0u << 13);          // 1: => Enable Bank Overflow interrupt

  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);

  OS_COM_INIT();                    // Initialize UART for embOSView
  OS_SetFastIntPriorityLimit(8u);   // Set Priority limit for "Fast" interrupts
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Please note:
*  This is basically the "core" of the idle loop.
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = CMCNT_0;
  //
  // Check if timer interrupt pending ...
  //
  if ((CMCSR_0 & (1u << 7)) != 0u) {
    Cnt = CMCNT_0;  // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000u);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

#if OS_UART_USED

/*********************************************************************
*
*		_IsrUartRxHandler()
*/
static void _IsrUartRxHandler(void) {
  if (SCIFx_SCFSR & (1u << SCFSR_RDF_BIT)) {
    OS_U8 Data;
    Data = SCIFx_SCFRDR;      // Get data
    SCIFx_SCFSR &= ~(1u << SCFSR_RDF_BIT);
    OS_OnRx(Data);             // Process data
  }
}

/*********************************************************************
*
*	_IsrUartTxHandler()
*/
static void _IsrUartTxHandler(void) {
  if (OS_OnTx()) {
    SCIFx_SCSCR &= ~(1u << SCSCR_TIE_BIT);      // Disable Tx-Int
  }
}

/*********************************************************************
*
*	_IsrUartErrHandler()
*/
static void _IsrUartErrHandler(void) {
  SCIFx_SCSCR  &= ~(0x50u);   // disable Rx and Rx-Int
  while (SCIFx_SCFSR & ((1u << 7) | (1u << 4) | (1u << 3) | (1u << 2))) {  // Any error (ER, BRK, FER, PER)?
    SCIFx_SCFSR    &= ~((1u << 7) | (1u << 4) | (1u << 3) | (1u << 2));    // Clear error and Rx data flags
  }
  while (SCIFx_SCLSR & (1u << 0)) { // Overrun error ?
    SCIFx_SCLSR &= ~(1u << 0);      // Clear error
  }
  SCIFx_SCSCR |= 0x50u;             // Reenable Rx and Rx-Int
}

/*********************************************************************
*
*       OS_COM_Init()
*/
void OS_COM_Init(void) {
  unsigned int i;
  OS_U16 PortMode;

  //
  // Configure the port pins connected to UART
  //
  PortMode  = PECRLx;
  PortMode &= ~(PE_MDRXD_MASK | PE_MDTXD_MASK); // Clear control setting
  PortMode |=  (0x03u << PE_MD_RXD_POS) | (0x03u << PE_MD_TXD_POS);  // Set SCIF-function (0x03)
  PECRLx    = PortMode;
  PEIORL   &= ~(1u << PE_RXD_PIN);  // Set data direction for RxD to input
  PEIORL   |=  (1u << PE_TXD_PIN);  // Set data direction for TxD to input
  //
  // Enable selected UART peripheral
  //
  STBCR4 &= ~(1u << SCIFx_MODULE_ID_BIT);
  //
  // Disable transmission, reception & serial interrupts
  //
  SCIFx_SCSCR = 0x0000u;
  //
  // Initialize the FIFO control register, disable FIFO
  //
  SCIFx_SCFCR = 0x0000u;
  //
  // Initializes the serial mode register
  //
  SCIFx_SCSMR = 0x0000u     // B15:B8 - Reserved - Set to 0
               | (0u << 7)  // B7 - Communication mode; 0: Asynchronous
               | (0u << 6)  // B6 - Character length;   0: 8 bit data
               | (0u << 5)  // B5 - Parity Enable;      0: No Parity
               | (0u << 4)  // B4 - Parity mode;        0: N/A (set to 0)
               | (0u << 3)  // B3 - Stop bit length;    0: 1 stop bit
               | (0u << 2)  // B2 - Reserved - Set to 0
               | (0u << 0)  // B1:B0 - Clock Select     0: Select Pphi as source clock
               ;
  //
  // Initialize Baudrate generator
  //
  SCIFx_SCBRR = BAUDDIVIDE;
  for(i=0u ; i < 10500u ; i++) {
    /* Setting delay (1 bit cycle) */
  }
  //
  // Initialize serial status register
  //
  SCIFx_SCFSR = 0x0000u;
  //
  // Set interrupt priority for SCI, enables interrupt
  //
  INTC_SCIFx_PRIO_REG = (0x01u << SCIFx_PRIO_POS);
  //
  // Enable transmission, reception and receive interrupts
  //
  SCIFx_SCSCR |= 0x0070u;
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send one character via UART.
*       Never call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  SCIFx_SCFTDR = c;
  SCIFx_SCFSR  &=~(1u << SCFSR_TDFE_BIT);  // Clear Tx Empty Flag
  SCIFx_SCSCR  |= (1u << SCSCR_TIE_BIT);   // Enable Tx-Int
}

/*********************************************************************
*
*       OS_ISR_rx()
*/
#pragma vector = 194
static __fast_interrupt __interrupt void OS_ISR_Rx(void) {
  #if ALLOW_NESTED_INTERRUPTS
    OS_CallNestableISR(_IsrUartRxHandler);
  #else
    OS_CallISR(_IsrUartRxHandler);
  #endif
}

/*********************************************************************
*
*       OS_ISR_tx()
*/
#pragma vector = 195
static __fast_interrupt __interrupt void OS_ISR_Tx(void) {
  #if ALLOW_NESTED_INTERRUPTS
    OS_CallNestableISR(_IsrUartTxHandler);
  #else
    OS_CallISR(_IsrUartTxHandler);
  #endif
}

/*********************************************************************
*
*       OS_ISR_RxErr()
*/
#pragma vector = 193
static __fast_interrupt __interrupt void OS_ISR_RxErr(void) {
  #if ALLOW_NESTED_INTERRUPTS
    OS_CallNestableISR(_IsrUartErrHandler);
  #else
    OS_CallISR(_IsrUartErrHandler);
  #endif
}

/*********************************************************************
*
*       OS_ISR_RxErr()
*/
#pragma vector = 192
static __fast_interrupt __interrupt void OS_ISR_RxErrB(void) {
  #if ALLOW_NESTED_INTERRUPTS
    OS_CallNestableISR(_IsrUartErrHandler);
  #else
    OS_CallISR(_IsrUartErrHandler);
  #endif
}

#else  /* UART for communication not used, define dummy functions */

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}
#endif

/*********************************************************************
*
*       __low_level_init()
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);    // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
#if OS_INIT_PLL
  //
  // Initialize the PLL, according to the procedure described in harware manual
  //
  WDT_WTCSR = 0xA518u;          // Stop WDT, b4..b3 always 1
  //
  // Watchdog should timeout after 10ms , initial peripheral frequency is 16,67MHz
  //
  WDT_WTCSR = 0xA51Du;          // b2..0: Divide input clock by 1024, b4..3: always 1
  WDT_WTCNT = (0x5A00u + 163u); // Overflow after 10 ms
  //
  // Initialize PLL configuration register, assuming external frequency = 16,67MHz, clock mode 0
  CPG_FRQCR = 0x0u        // b15..b14: Reserved - set to 0
            | (0u << 12)  // b13..b12: Clock output enable, set to 0
            | (0u << 10)  // b11..b10: Reserved - set to 0
            | (1u << 8)   // b8..b9: b Multiplier, 01=> 12, => PLLout = 16,67MHz * 12 = 200MHz
            | (0u << 5)   // b7..b5:   Reserved - set to 0
            | (0u << 4)   // Internal division ratio, 0 => 1
            | (0u << 3)   // b3 :      Reserved - set to 0
            | (4u << 0)   // b2..b0:   Peripheral clock division, 4 => 6, peripheral clock = 33MHz
            ;
#endif
  //
  // Enable instruction and data cache, write-back mode and flush caches
  //
  CCR1 = 0x909;
  //
  // Perform additional initialization here if required
  //


  //
  // Ensure segments are initialized, always return 1
  //
  return 1;
}

/****** End Of File *************************************************/
