/* Adapted for use with IAR Embedded Workbench */
/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name     : hwsetup.c
* Version       : 1.00
* Device(s)     : SH7269
* Tool-Chain    : HEW, SuperH RISC engine Standard Toolchain v9.4.0.0
* H/W Platform  : RSK2+SH7269
* Description   : Contains hardware setup functions
*******************************************************************************
* History       : DD.MM.YYYY Version Description
*      : 08.03.2010 Adapted to IAR Embedded Workbench
*               : 03.03.2012 1.00    First Release
******************************************************************************/

/******************************************************************************
System Includes
******************************************************************************/
/* This header file provides access to intrinsic functions useful for
   configuring and controlling the interrupts. */
#include <intrinsics.h>


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

/*  Header file provides a structure to access on-chip I/O registers. */
#include "io7268.h"
/* Header file provides prototypes for hardware configuration functions. */
#include "hwsetup.h"
/* Header file provides prototypes for external bus initialization */
#include "bus_init.h"
/* Header file provides rte type definitions. */
#include "stdint.h"

/******************************************************************************
Private global variables and functions
******************************************************************************/
/* These functions are private so their prototypes are defined locally */
static void configure_operating_frequency (void);
static void enable_peripheral_modules (void);
static void configure_port_pins (void);

/******************************************************************************
User Program Code
******************************************************************************/
#define CCR1    (*(unsigned int*)0xFFFC1000u)  // Cache Control Register 1 (CCR1)

/******************************************************************************
* Function Name : HardwareSetup
* Description   : Sets up the hardware.
*                 This function calls the hardware initialization functions to
*                 configure the CPU operating frequency, port pins & required
*                 on-chip modules in order to setup the RSK for the main
*                 application.
* Argument      : none
* Return value  : none
******************************************************************************/

void HardwareSetup (void)
{

    /* Configures CPU clock    */
    configure_operating_frequency();

    /* configure the external Memory */
    configure_bus_and_memory();

    /* Configures port pins    */
    configure_port_pins();

    /* Enables required on-chip peripherals    */
    enable_peripheral_modules();
}

/******************************************************************************
End of function HardwareSetup
******************************************************************************/

/******************************************************************************
* Function Name : configure_operating_frequency
* Description   : Sets up operating speed and configures the main & sub-clock
*                 and USB clock
* Argument      : none
* Return value  : none
******************************************************************************/

static void configure_operating_frequency (void)
{

    /*    Configure Frequency Control Register
        b0        - Reserved      - Set to 1
        b1        - Reserved      - Set to 0
        b2        - Reserved      - Set to 1
        b3        - Reserved      - Set to 0
        b5:b4     - BFC1:IFC0     - Set to 00 Bus Clock frequency division ratio
        b7:b6     - Reserved      - Set to 00
        b9:b8     - IFC1:IFC0     - set to 00 CPU Clock Frequency division ratio
        b11:b10   - Reserved      - set to 000
        b13:b12   - CKOEN1:CKOEN0 - set to 01
        b14       - CKOEN2        - set to 0
        b15       - Reserved      - set to 0

        Multiplication Ratio PLL(x20),I:B:P1:P0= 20:10:5:5/2
        CKIO:CKIO:Output at time usually,Output when bus right is opened
        Clockin = 13.34MHz, CKIO = 66.67MHz
        I Clock = 266.66MHz, B Clock = 133.33MHz,
        P1 Clock = 66.67MHz
        P0 Clock = 33.33MHz    */

    CPG.FRQCR.WORD = OPERATING_FREQUENCY;
  //
  // Enable instruction and data cache, write-back mode and flush caches
  //
  CCR1 = 0x909;
}

/******************************************************************************
End of function configure_operating_frequency
******************************************************************************/

/******************************************************************************
* Function Name : configure_port_pins
* Description   : Configures MCU port pins. This function configures the port
*                 pins so that it can be used by the respective modules. It
*                 also configures the unused port pins as output low.
* Argument      : none
* Return value  : none
******************************************************************************/

static void configure_port_pins (void)
{
}

/******************************************************************************
End of function configure_port_pins
******************************************************************************/

/******************************************************************************
* Function Name : enable_peripheral_modules
* Description   : Enables Peripheral Modules before use
* Argument      : none
* Return value  : none
* Note          : When writing to the register related to power-down modes,
*                 the CPU, after executing a write instruction, executes the
*                 next instruction without waiting for the write operation to
*                 complete. Therefore, to reflect the change specified by
*                 writing to the register while the next instruction is
*                 executed, insert a dummy read of the same register between
*                 the register write instruction and the next instruction.
******************************************************************************/

static void enable_peripheral_modules (void)
{
    volatile uint8_t dummy;

    /* Enable on-chip RAM */
    CPG.SYSCR1.BYTE = OC_RAM_ENABLE_MODE;
    dummy = CPG.SYSCR1.BYTE;

    /* Enable ADC module */
    CPG.STBCR3.BIT.MSTP32 = 0;
    dummy = CPG.STBCR3.BYTE;

    /* Enable MTU2 module */
    CPG.STBCR3.BIT.MSTP35 = 0;
    dummy = CPG.STBCR3.BYTE;

    /* Enable IIC module channel 2 (SCL2 & SLA2) */
    CPG.STBCR5.BIT.MSTP55 = 0;
    dummy = CPG.STBCR5.BYTE;
}

/******************************************************************************
End of function enable_peripheral_modules
******************************************************************************/

/******************************************************************************
End of File
******************************************************************************/


